/*
 * Copyright (C) 2001 USAGI/WIDE Project.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * This is a parse routine for a message of SADB_UPDATE.
 */

#ifdef PFKEY_DEBUG
#define DEBUG 1
#endif

#ifdef MODULE
#include <linux/module.h>
#  ifdef MODVERSIONS
#  include <linux/modversions.h>
#  endif
#endif

#include <linux/config.h>
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/malloc.h>
#include <linux/socket.h>
#include <linux/in.h>
#include <linux/in6.h>
#include <linux/random.h>

#include <net/pfkeyv2.h>
#include <net/sadb.h>

#include "pfkey_v2_msg.h"
#include "sadb_utils.h" /* esptoa authtoa */
			/* sockaddrtoa ultoa */
#define BUFSIZE 64

int sadb_msg_update_parse(struct sock *sk, struct sadb_msg* msg, struct sadb_msg **reply)
{
	int error = 0;
	__u32 spi = 0;
	__u64 ext_exist = 0;
	struct sadb_ext *ext_msgs[SADB_EXT_MAX+1];
	struct sadb_sa *sa = NULL;
	struct sadb_address *src = NULL;
	struct sadb_address *dst = NULL;
	struct sadb_address *pxy = NULL;
	struct ipsec_sa *sa_entry = NULL;
	struct sockaddr_storage saddr, daddr, paddr;

	if(!msg){
		pr_debug("sadb_msg_update_parse: msg==null\n");
		error = -EINVAL;
		goto err;
	}

	error = sadb_msg_detect_ext(msg, ext_msgs, &ext_exist);

	
	if(ext_exist&(1<<SADB_EXT_SA) &&
		ext_exist&(1<<SADB_EXT_ADDRESS_SRC) &&
		ext_exist&(1<<SADB_EXT_ADDRESS_DST) &&
		(ext_exist&(1<<SADB_EXT_KEY_AUTH) ||
		 ext_exist&(1<<SADB_EXT_KEY_ENCRYPT)))
	{
		
		memset(&saddr, 0, sizeof(struct sockaddr_storage));
		memset(&daddr, 0, sizeof(struct sockaddr_storage));
		memset(&paddr, 0, sizeof(struct sockaddr_storage));

		src = (struct sadb_address*)ext_msgs[SADB_EXT_ADDRESS_SRC];
		dst = (struct sadb_address*)ext_msgs[SADB_EXT_ADDRESS_DST];

		spi = sa->sadb_sa_spi;

		error = sadb_address_to_sockaddr( src, (struct sockaddr*)&saddr);

		if(error){
			pr_debug("sadb_msg_add_parse: src translation failed\n");
			goto err;
		}
		error = sadb_address_to_sockaddr( dst, (struct sockaddr*)&daddr);
		if(error){
			pr_debug("sadb_msg_add_parse: dst translation failed\n");
			goto err;
		}

		if(ext_exist&(1<<SADB_EXT_ADDRESS_PROXY)){
			pxy = (struct sadb_address*)ext_msgs[SADB_EXT_ADDRESS_PROXY];
			error = sadb_address_to_sockaddr( pxy, (struct sockaddr*)&paddr);
			if(error){
				pr_debug("sadb_msg_add_parse: pxy translation falied\n");
				goto err;
			}
		}

		if(sa->sadb_sa_auth && !(ext_exist&(1<<SADB_EXT_KEY_AUTH))){
			pr_debug("sadb_msg_add_parse: sa has auth algo but there is no key for auth\n");
			error = -EINVAL;
			goto err;
		}

		if(sa->sadb_sa_encrypt && !(ext_exist&(1<<SADB_EXT_KEY_ENCRYPT))){
			pr_debug("pfkey: sadb_msg_add_parse: sa has esp algo but there is no key for esp\n");
			error = -EINVAL;
			goto err;
		}

		if(ext_exist&(1<<SADB_EXT_ADDRESS_PROXY)){
			error = sadb_find_by_address_proto_spi((struct sockaddr*)&saddr,
						 (struct sockaddr*)&daddr,
						 (struct sockaddr*)&paddr,
						 msg->sadb_msg_type,
						 spi,
						 &sa_entry);
		}else{
			error = sadb_find_by_address_proto_spi((struct sockaddr*)&saddr,
						 (struct sockaddr*)&daddr,
						 NULL, /* ignore proxy */
						 msg->sadb_msg_type,
						 spi,
						 &sa_entry);

		}

		if(error){
			pr_debug("sadb_msg_update_parse: could not find sa\n");
			goto err;
		}

		switch(sa_entry->state){

		case SADB_SASTATE_LARVAL:

		break;

		case SADB_SASTATE_MATURE:
		case SADB_SASTATE_DYING:

		break;

		case SADB_SASTATE_DEAD:
		default:

		}

	}
	/* 0xFFFFFFFFFFFFFCFF is a mask for auth key and esp key */
//	sadb_msg_reply_build(msg, ext_msgs, ext_exist&0xFFFFFFFFFFFFFCFF, reply);
	sadb_release_ref(sa_entry);
	return error;
err:
//	sadb_msg_reply_build2(msg, error, reply);
	sadb_release_ref(sa_entry);
	return error;
}

